@@ -30,6 +30,7 @@
 #include "Extender.h"
 #include "DirectorExtension.h"
 #include "SciTE.h"
+#include "FilePath.h"
 #include "SciTEBase.h"
 
 static ExtensionAPI *host = 0;
@@ -56,7 +57,7 @@ static void SendDirector(const char *verb, const char *arg = 0) {
 		addressedMessage += ":";
 		if (arg)
 			addressedMessage += arg;
-		char *slashedMessage = Slash(addressedMessage.c_str());
+		char *slashedMessage = Slash(addressedMessage.c_str(), false);
 		if (slashedMessage) {
 			COPYDATASTRUCT cds;
 			cds.dwData = 0;
@@ -160,8 +161,8 @@ bool DirectorExtension::Initialise(ExtensionAPI *host_) {
 	if (!wReceiver)
 		::exit(FALSE);
 	// Make the frame window handle available so the director can activate it.
-	::SetWindowLong(wReceiver, GWL_USERDATA,
-		reinterpret_cast<LONG>(((SciTEBase*)host)->GetID()));
+	::SetWindowLongPtr(wReceiver, GWLP_USERDATA,
+		reinterpret_cast<LONG_PTR>(((SciTEBase*)host)->GetID()));
 	CheckEnvironment(host);
 	return true;
 }
@@ -196,7 +197,7 @@ bool DirectorExtension::OnSwitchFile(const char *path) {
 		::SendDirector("switched", path);
 	}
 	return false;
-};
+}
 
 bool DirectorExtension::OnSave(const char *path) {
 	CheckEnvironment(host);
