@@ -46,7 +46,7 @@ static void PlayThisSound(
 	if (soundFreq == 0) {	// sound is probably a path
 		if (!hMM) {
 			// Load the DLL only if needed (may be slow on some systems)
-			hMM = ::LoadLibrary("WINMM.DLL");
+			hMM = ::LoadLibrary(TEXT("WINMM.DLL"));
 		}
 
 		if (hMM) {
@@ -75,56 +75,11 @@ static void PlayThisSound(
 	}
 }
 
-// Handle Unicode controls (assume strings to be UTF-8 on Windows NT)
-
-// This is a reasonable buffer size for dialog box text conversions
-#define CTL_TEXT_BUF 512
-
-SString GetDlgItemText2(HWND hDlg, int idItem) {
-	SString result = "";
-	WCHAR wsz[CTL_TEXT_BUF];
-	char	msz[CTL_TEXT_BUF];
-
-	if (::IsWindowUnicode(GetDlgItem(hDlg, idItem))) {
-		if (::GetDlgItemTextW(hDlg, idItem, wsz, CTL_TEXT_BUF)) {
-			if (::WideCharToMultiByte(CP_UTF8, 0, wsz, -1, msz, CTL_TEXT_BUF, NULL, NULL))
-				result = msz;
-		}
-	} else {
-		if (::GetDlgItemTextA(hDlg, idItem, msz, CTL_TEXT_BUF))
-			result = msz;
-	}
-
-	//::MessageBox(GetFocus(),result.c_str(),"GetDlgItemText2():out",0);
-
-	return result;
-}
-
-BOOL SetDlgItemText2(HWND hDlg, int idItem, LPCSTR pmsz) {
-	BOOL bSuccess = FALSE;
-	WCHAR wsz[CTL_TEXT_BUF];
-
-	if (!pmsz || *pmsz == 0)
-		return ::SetDlgItemTextA(hDlg, idItem, "");
-
-	//::MessageBox(GetFocus(),pmsz,"SetDlgItemText2():in",0);
-
-	if (::IsWindowUnicode(::GetDlgItem(hDlg, idItem))) {
-		if (::MultiByteToWideChar(CP_UTF8, 0, pmsz, -1, wsz, CTL_TEXT_BUF)) {
-			bSuccess = ::SetDlgItemTextW(hDlg, idItem, wsz);
-		}
-	} else {
-		bSuccess = ::SetDlgItemTextA(hDlg, idItem, pmsz);
-	}
-
-	return bSuccess;
-}
-
 static SciTEWin *Caller(HWND hDlg, UINT message, LPARAM lParam) {
 	if (message == WM_INITDIALOG) {
-		SetWindowLong(hDlg, DWL_USER, lParam);
+		SetWindowLongPtr(hDlg, DWLP_USER, lParam);
 	}
-	return reinterpret_cast<SciTEWin*>(GetWindowLong(hDlg, DWL_USER));
+	return reinterpret_cast<SciTEWin*>(GetWindowLongPtr(hDlg, DWLP_USER));
 }
 
 void SciTEWin::WarnUser(int warnID) {
@@ -219,23 +174,10 @@ int SciTEWin::DoDialog(HINSTANCE hInst, const char *resName, HWND hWnd, DLGPROC
 	return result;
 }
 
-bool SciTEWin::OpenDialog(const char *filter) {
+bool SciTEWin::OpenDialog(FilePath directory, const char *filter) {
+	enum {maxBufferSize=2048};
 
-	char openName[2048]; // maximum common dialog buffer size (says mfc..)
-	*openName = '\0';
-
-	OPENFILENAME ofn = {
-	                       sizeof(OPENFILENAME), 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
-	                   };
-	ofn.hwndOwner = MainHWND();
-	ofn.hInstance = hInstance;
-	ofn.lpstrFile = openName;
-	ofn.nMaxFile = sizeof(openName);
-	SString openFilter;
-	if (filter)
-		openFilter = filter;
-	else
-		openFilter = props.GetExpanded("open.filter");
+	SString openFilter = filter;
 	if (openFilter.length()) {
 		openFilter.substitute('|', '\0');
 		size_t start = 0;
@@ -256,18 +198,31 @@ bool SciTEWin::OpenDialog(const char *filter) {
 			}
 		}
 	}
-	ofn.lpstrFilter = openFilter.c_str();
+
 	if (!openWhat[0]) {
 		strcpy(openWhat, LocaliseString("Custom Filter").c_str());
 		openWhat[strlen(openWhat) + 1] = '\0';
 	}
+
+	bool succeeded = false;
+	char openName[maxBufferSize]; // maximum common dialog buffer size (says mfc..)
+	openName[0] = '\0';
+	
+	OPENFILENAMEA ofn = {
+	       sizeof(OPENFILENAME), 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
+	};
+	ofn.hwndOwner = MainHWND();
+	ofn.hInstance = hInstance;
+	ofn.lpstrFile = openName;
+	ofn.nMaxFile = maxBufferSize;
+	ofn.lpstrFilter = openFilter.c_str();
 	ofn.lpstrCustomFilter = openWhat;
 	ofn.nMaxCustFilter = sizeof(openWhat);
 	ofn.nFilterIndex = filterDefault;
 	SString translatedTitle = LocaliseString("Open File");
 	ofn.lpstrTitle = translatedTitle.c_str();
 	if (props.GetInt("open.dialog.in.file.directory")) {
-		ofn.lpstrInitialDir = dirName;
+		ofn.lpstrInitialDir = directory.AsFileSystem();
 	}
 	ofn.Flags = OFN_HIDEREADONLY;
 
@@ -278,58 +233,32 @@ bool SciTEWin::OpenDialog(const char *filter) {
 		    OFN_ALLOWMULTISELECT;
 	}
 	if (::GetOpenFileName(&ofn)) {
+		succeeded = true;
 		filterDefault = ofn.nFilterIndex;
-		//Platform::DebugPrintf("Open: <%s>\n", openName);
-		// find char pos after first Delimiter
-		char* p = openName;
-		while (*p != '\0')
-			++p;
-		++p;
 		// if single selection then have path+file
-		if ((p - openName) > ofn.nFileOffset) {
+		if (strlen(openName) > static_cast<size_t>(ofn.nFileOffset)) {
 			Open(openName);
 		} else {
-			SString path(openName);
-			size_t len = path.length();
-			if ((len > 0) && (path[len - 1] != '\\'))
-				path += "\\";
-			while (*p != '\0') {
+			FilePath directory(openName);
+			char *p = openName + strlen(openName) + 1;
+			while (*p) {
 				// make path+file, add it to the list
-				SString file = path;
-				file += p;
-				Open(file.c_str());
+				Open(FilePath(directory, FilePath(p)));
 				// goto next char pos after \0
-				while (*p != '\0')
-					++p;
-				++p;
+				p += strlen(p) + 1;
 			}
 		}
-	} else {
-		return false;
 	}
-	return true;
+	return succeeded;
 }
 
-SString SciTEWin::ChooseSaveName(const char *title, const char *filter, const char *ext) {
-	SString path;
+FilePath SciTEWin::ChooseSaveName(FilePath directory, const char *title, const char *filter, const char *ext) {
+	FilePath path;
 	if (0 == dialogsOnScreen) {
 		char saveName[MAX_PATH] = "";
-		strcpy(saveName, fileName);
-		if (ext) {
-			char *cpDot = strrchr(saveName, '.');
-			int keepExt = props.GetInt("export.keep.ext", 0);
-			if (cpDot != NULL) {
-				if (keepExt == 0) {
-					strcpy(cpDot, ext);
-				} else if (keepExt == 1) {
-					strcat(saveName, ext);
-				} else if (keepExt == 2) {
-					*cpDot = '_';
-					strcat(saveName, ext);
-				}
-			} else {
-				strcat(saveName, ext);
-			}
+		FilePath savePath = SaveName(ext);
+		if (!savePath.IsUntitled()) {
+			strcpy(saveName, savePath.AsFileSystem());
 		}
 		OPENFILENAME ofn = {
 		                       sizeof(OPENFILENAME), 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
@@ -342,7 +271,7 @@ SString SciTEWin::ChooseSaveName(const char *title, const char *filter, const ch
 		ofn.lpstrTitle = translatedTitle.c_str();
 		ofn.Flags = OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT;
 		ofn.lpstrFilter = filter;
-		ofn.lpstrInitialDir = dirName;
+		ofn.lpstrInitialDir = directory.AsFileSystem();
 
 		dialogsOnScreen++;
 		if (::GetSaveFileName(&ofn)) {
@@ -354,10 +283,10 @@ SString SciTEWin::ChooseSaveName(const char *title, const char *filter, const ch
 }
 
 bool SciTEWin::SaveAsDialog() {
-	SString path = ChooseSaveName("Save File");
-	if (path.length()) {
+	FilePath path = ChooseSaveName(filePath.Directory(), "Save File");
+	if (path.IsSet()) {
 		//Platform::DebugPrintf("Save: <%s>\n", openName);
-		SetFileName(path.c_str(), false); // don't fix case
+		SetFileName(path, false); // don't fix case
 		Save();
 		ReadProperties();
 
@@ -365,56 +294,56 @@ bool SciTEWin::SaveAsDialog() {
 		SendEditor(SCI_COLOURISE, 0, -1);
 		wEditor.InvalidateAll();
 		if (extender)
-			extender->OnSave(fullPath);
+			extender->OnSave(filePath.AsFileSystem());
 		return true;
 	}
 	return false;
 }
 
 void SciTEWin::SaveACopy() {
-	SString path = ChooseSaveName("Save a Copy");
-	if (path.length()) {
-		SaveBuffer(path.c_str());
+	FilePath path = ChooseSaveName(filePath.Directory(), "Save a Copy");
+	if (path.IsSet()) {
+		SaveBuffer(path);
 	}
 }
 
 void SciTEWin::SaveAsHTML() {
-	SString path = ChooseSaveName("Export File As HTML",
+	FilePath path = ChooseSaveName(filePath.Directory(), "Export File As HTML",
 	                              "Web (.html;.htm)\0*.html;*.htm\0", ".html");
-	if (path.length()) {
-		SaveToHTML(path.c_str());
+	if (path.IsSet()) {
+		SaveToHTML(path);
 	}
 }
 
 void SciTEWin::SaveAsRTF() {
-	SString path = ChooseSaveName("Export File As RTF",
+	FilePath path = ChooseSaveName(filePath.Directory(), "Export File As RTF",
 	                              "RTF (.rtf)\0*.rtf\0", ".rtf");
-	if (path.length()) {
-		SaveToRTF(path.c_str());
+	if (path.IsSet()) {
+		SaveToRTF(path);
 	}
 }
 
 void SciTEWin::SaveAsPDF() {
-	SString path = ChooseSaveName("Export File As PDF",
+	FilePath path = ChooseSaveName(filePath.Directory(), "Export File As PDF",
 	                              "PDF (.pdf)\0*.pdf\0", ".pdf");
-	if (path.length()) {
-		SaveToPDF(path.c_str());
+	if (path.IsSet()) {
+		SaveToPDF(path);
 	}
 }
 
 void SciTEWin::SaveAsTEX() {
-	SString path = ChooseSaveName("Export File As TeX",
+	FilePath path = ChooseSaveName(filePath.Directory(), "Export File As TeX",
 	                              "TeX (.tex)\0*.tex\0", ".tex");
-	if (path.length()) {
-		SaveToTEX(path.c_str());
+	if (path.IsSet()) {
+		SaveToTEX(path);
 	}
 }
 
 void SciTEWin::SaveAsXML() {
-	SString path = ChooseSaveName("Export File As XML",
+	FilePath path = ChooseSaveName(filePath.Directory(), "Export File As XML",
 	                              "TeX (.xml)\0*.xml\0", ".xml");
-	if (path.length()) {
-		SaveToXML(path.c_str());
+	if (path.IsSet()) {
+		SaveToXML(path);
 	}
 }
 
@@ -794,28 +723,110 @@ void SciTEWin::PrintSetup() {
 
 extern bool IsWindowsNT();
 
-static void FillComboFromMemory(HWND combo, const ComboMemory &mem, bool useTop = false) {
-	::SendMessage(combo, CB_RESETCONTENT, 0, 0);
-	if (::IsWindowUnicode(combo)) {
-		for (int i = 0; i < mem.Length(); i++) {
-			//Platform::DebugPrintf("Combo[%0d] = %s\n", i, mem.At(i).c_str());
-			WCHAR wszBuf[CTL_TEXT_BUF];
-			::MultiByteToWideChar(CP_UTF8, 0, mem.At(i).c_str(), -1, wszBuf,
-			                    CTL_TEXT_BUF);
-			::SendMessageW(combo, CB_ADDSTRING, 0,
-			               reinterpret_cast<LPARAM>(wszBuf));
+// This is a reasonable buffer size for dialog box text conversions
+#define CTL_TEXT_BUF 512
+
+class Dialog {
+	HWND hDlg;
+public:
+
+	Dialog(HWND hDlg_) : hDlg(hDlg_) {
+	}
+
+	HWND Item(int id) {
+		return ::GetDlgItem(hDlg, id);
+	}
+
+	void Enable(int id, bool enable) {
+		::EnableWindow(Item(id), enable);
+	}
+
+	SString ItemText(int id) {
+		HWND wT = Item(id);
+		int len = ::GetWindowTextLength(wT);
+		SBuffer itemText(len);
+		if (len > 0) {
+			::GetDlgItemText(hDlg, id, itemText.ptr(), len + 1);
 		}
-	} else {
-		for (int i = 0; i < mem.Length(); i++) {
-			//Platform::DebugPrintf("Combo[%0d] = %s\n", i, mem.At(i).c_str());
-			::SendMessage(combo, CB_ADDSTRING, 0,
-			              reinterpret_cast<LPARAM>(mem.At(i).c_str()));
+		return SString(itemText);
+	}
+
+	void SetItemText(int id, const char *s) {
+		::SetDlgItemText(hDlg, id, s);
+	}
+
+	// Handle Unicode controls (assume strings to be UTF-8 on Windows NT)
+	
+	SString ItemTextU(int id) {
+		SString result = "";
+		char	msz[CTL_TEXT_BUF];
+	
+		if (::IsWindowUnicode(Item(id))) {
+			WCHAR wsz[CTL_TEXT_BUF];
+			if (::GetDlgItemTextW(hDlg, id, wsz, CTL_TEXT_BUF)) {
+				if (::WideCharToMultiByte(CP_UTF8, 0, wsz, -1, msz, CTL_TEXT_BUF, NULL, NULL))
+					result = msz;
+			}
+		} else {
+			if (::GetDlgItemTextA(hDlg, id, msz, CTL_TEXT_BUF))
+				result = msz;
+		}
+	
+		return result;
+	}
+	
+	BOOL SetItemTextU(int id, LPCSTR pmsz) {
+		BOOL bSuccess = FALSE;
+		WCHAR wsz[CTL_TEXT_BUF];
+	
+		if (!pmsz || *pmsz == 0)
+			return ::SetDlgItemTextA(hDlg, id, "");
+	
+		if (::IsWindowUnicode(Item(id))) {
+			if (::MultiByteToWideChar(CP_UTF8, 0, pmsz, -1, wsz, CTL_TEXT_BUF)) {
+				bSuccess = ::SetDlgItemTextW(hDlg, id, wsz);
+			}
+		} else {
+			bSuccess = ::SetDlgItemTextA(hDlg, id, pmsz);
+		}
+	
+		return bSuccess;
+	}
+
+	void SetCheck(int id, bool value) {
+		::SendMessage(::GetDlgItem(hDlg, id), BM_SETCHECK, 
+			value ? BST_CHECKED : BST_UNCHECKED, 0);
+	}
+
+	bool Checked(int id) {
+		return BST_CHECKED == ::SendMessage(::GetDlgItem(hDlg, id), BM_GETCHECK, 0, 0);
+	}
+
+	void FillComboFromMemory(int id, const ComboMemory &mem, bool useTop = false) {
+		HWND combo = Item(id);
+		::SendMessage(combo, CB_RESETCONTENT, 0, 0);
+		if (::IsWindowUnicode(combo)) {
+			for (int i = 0; i < mem.Length(); i++) {
+				//Platform::DebugPrintf("Combo[%0d] = %s\n", i, mem.At(i).c_str());
+				WCHAR wszBuf[CTL_TEXT_BUF];
+				::MultiByteToWideChar(CP_UTF8, 0, mem.At(i).c_str(), -1, wszBuf,
+						    CTL_TEXT_BUF);
+				::SendMessageW(combo, CB_ADDSTRING, 0,
+					       reinterpret_cast<LPARAM>(wszBuf));
+			}
+		} else {
+			for (int i = 0; i < mem.Length(); i++) {
+				//Platform::DebugPrintf("Combo[%0d] = %s\n", i, mem.At(i).c_str());
+				::SendMessage(combo, CB_ADDSTRING, 0,
+					      reinterpret_cast<LPARAM>(mem.At(i).c_str()));
+			}
+		}
+		if (useTop) {
+			::SendMessage(combo, CB_SETCURSEL, 0, 0);
 		}
 	}
-	if (useTop) {
-		::SendMessage(combo, CB_SETCURSEL, 0, 0);
-	}
-}
+
+};
 
 static void FillComboFromProps(HWND combo, PropSet &props) {
 	char *key;
@@ -828,58 +839,30 @@ static void FillComboFromProps(HWND combo, PropSet &props) {
 	}
 }
 
-static SString GetItemText(HWND hDlg, int id) {
-	HWND wT = ::GetDlgItem(hDlg, id);
-	int len = ::GetWindowTextLength(wT);
-	SBuffer itemText(len);
-	if (len > 0) {
-		::GetDlgItemText(hDlg, id, itemText.ptr(), len + 1);
-	}
-	return SString(itemText);
-}
-
 BOOL SciTEWin::FindMessage(HWND hDlg, UINT message, WPARAM wParam) {
 	// Avoid getting dialog items before set up or during tear down.
 	if (WM_SETFONT == message || WM_NCDESTROY == message)
 		return FALSE;
-	HWND wFindWhat = ::GetDlgItem(hDlg, IDFINDWHAT);
-	HWND wWholeWord = ::GetDlgItem(hDlg, IDWHOLEWORD);
-	HWND wMatchCase = ::GetDlgItem(hDlg, IDMATCHCASE);
-	HWND wRegExp = ::GetDlgItem(hDlg, IDREGEXP);
-	HWND wWrap = ::GetDlgItem(hDlg, IDWRAP);
-	HWND wUnSlash = ::GetDlgItem(hDlg, IDUNSLASH);
-	HWND wFindInStyle = ::GetDlgItem(hDlg, IDFINDINSTYLE);
-	HWND wFindStyle = ::GetDlgItem(hDlg, IDFINDSTYLE);
-	HWND wUp = ::GetDlgItem(hDlg, IDDIRECTIONUP);
-	HWND wDown = ::GetDlgItem(hDlg, IDDIRECTIONDOWN);
+	Dialog dlg(hDlg);
+	HWND wFindStyle = dlg.Item(IDFINDSTYLE);
 
 	switch (message) {
 
 	case WM_INITDIALOG:
 		LocaliseDialog(hDlg);
-		FillComboFromMemory(wFindWhat, memFinds);
-		SetDlgItemText2(hDlg, IDFINDWHAT, findWhat.c_str());
-		if (wholeWord)
-			::SendMessage(wWholeWord, BM_SETCHECK, BST_CHECKED, 0);
-		if (matchCase)
-			::SendMessage(wMatchCase, BM_SETCHECK, BST_CHECKED, 0);
-		if (regExp)
-			::SendMessage(wRegExp, BM_SETCHECK, BST_CHECKED, 0);
-		if (wrapFind)
-			::SendMessage(wWrap, BM_SETCHECK, BST_CHECKED, 0);
-		if (unSlash)
-			::SendMessage(wUnSlash, BM_SETCHECK, BST_CHECKED, 0);
-		if (findInStyle)
-			::SendMessage(wFindInStyle, BM_SETCHECK, BST_CHECKED, 0);
-		else
-			::EnableWindow(wFindStyle, false);
+		dlg.FillComboFromMemory(IDFINDWHAT, memFinds);
+		dlg.SetItemTextU(IDFINDWHAT, findWhat.c_str());
+		::SendDlgItemMessage(hDlg, IDFINDWHAT, CB_LIMITTEXT, CTL_TEXT_BUF - 1, 0);
+		dlg.SetCheck(IDWHOLEWORD, wholeWord);
+		dlg.SetCheck(IDMATCHCASE, matchCase);
+		dlg.SetCheck(IDREGEXP, regExp);
+		dlg.SetCheck(IDWRAP, wrapFind);
+		dlg.SetCheck(IDUNSLASH, unSlash);
+		dlg.SetCheck(IDFINDSTYLE, findInStyle);
+		dlg.Enable(IDFINDSTYLE, findInStyle);
 		::SendMessage(wFindStyle, EM_LIMITTEXT, 3, 1);
 		::SetDlgItemInt(hDlg, IDFINDSTYLE, SendEditor(SCI_GETSTYLEAT, SendEditor(SCI_GETCURRENTPOS)), FALSE);
-		if (reverseFind) {
-			::SendMessage(wUp, BM_SETCHECK, BST_CHECKED, 0);
-		} else {
-			::SendMessage(wDown, BM_SETCHECK, BST_CHECKED, 0);
-		}
+		dlg.SetCheck(reverseFind ? IDDIRECTIONUP : IDDIRECTIONDOWN, true);
 		return TRUE;
 
 	case WM_CLOSE:
@@ -893,24 +876,17 @@ BOOL SciTEWin::FindMessage(HWND hDlg, UINT message, WPARAM wParam) {
 			return FALSE;
 		} else if ( (ControlIDOfCommand(wParam) == IDOK) ||
 		            (ControlIDOfCommand(wParam) == IDMARKALL) ) {
-			findWhat = GetDlgItemText2(hDlg, IDFINDWHAT);
+			findWhat = dlg.ItemTextU(IDFINDWHAT);
 			props.Set("find.what", findWhat.c_str());
 			memFinds.Insert(findWhat.c_str());
-			wholeWord = BST_CHECKED ==
-			            ::SendMessage(wWholeWord, BM_GETCHECK, 0, 0);
-			matchCase = BST_CHECKED ==
-			            ::SendMessage(wMatchCase, BM_GETCHECK, 0, 0);
-			regExp = BST_CHECKED ==
-			         ::SendMessage(wRegExp, BM_GETCHECK, 0, 0);
-			wrapFind = BST_CHECKED ==
-			           ::SendMessage(wWrap, BM_GETCHECK, 0, 0);
-			unSlash = BST_CHECKED ==
-			          ::SendMessage(wUnSlash, BM_GETCHECK, 0, 0);
-			findInStyle = BST_CHECKED ==
-			              ::SendMessage(wFindInStyle, BM_GETCHECK, 0, 0);
-			findStyle = atoi(GetDlgItemText2(hDlg, IDFINDSTYLE).c_str());
-			reverseFind = BST_CHECKED ==
-			              ::SendMessage(wUp, BM_GETCHECK, 0, 0);
+			wholeWord = dlg.Checked(IDWHOLEWORD);
+			matchCase = dlg.Checked(IDMATCHCASE);
+			regExp = dlg.Checked(IDREGEXP);
+			wrapFind = dlg.Checked(IDWRAP);
+			unSlash = dlg.Checked(IDUNSLASH);
+			findInStyle = dlg.Checked(IDFINDINSTYLE);
+			findStyle = atoi(dlg.ItemTextU(IDFINDSTYLE).c_str());
+			reverseFind = dlg.Checked(IDDIRECTIONUP);
 			::EndDialog(hDlg, IDOK);
 			wFindReplace.Destroy();
 			if (ControlIDOfCommand(wParam) == IDMARKALL){
@@ -919,7 +895,8 @@ BOOL SciTEWin::FindMessage(HWND hDlg, UINT message, WPARAM wParam) {
 			FindNext(reverseFind);
 			return TRUE;
 		} else if (ControlIDOfCommand(wParam) == IDFINDINSTYLE) {
-			::EnableWindow(wFindStyle, BST_CHECKED == ::SendMessage(wFindInStyle, BM_GETCHECK, 0, 0));
+			findInStyle = dlg.Checked(IDFINDINSTYLE);
+			dlg.Enable(IDFINDSTYLE, findInStyle);
 			return TRUE;
 		}
 	}
@@ -935,33 +912,22 @@ BOOL SciTEWin::HandleReplaceCommand(int cmd) {
 	if (!wFindReplace.GetID())
 		return TRUE;
 	HWND hwndFR = reinterpret_cast<HWND>(wFindReplace.GetID());
-	HWND wWholeWord = ::GetDlgItem(hwndFR, IDWHOLEWORD);
-	HWND wMatchCase = ::GetDlgItem(hwndFR, IDMATCHCASE);
-	HWND wRegExp = ::GetDlgItem(hwndFR, IDREGEXP);
-	HWND wWrap = ::GetDlgItem(hwndFR, IDWRAP);
-	HWND wUnSlash = ::GetDlgItem(hwndFR, IDUNSLASH);
-	HWND wFindInStyle = ::GetDlgItem(hwndFR, IDFINDINSTYLE);
+	Dialog dlg(hwndFR);
 
 	if ((cmd == IDOK) || (cmd == IDREPLACE) || (cmd == IDREPLACEALL) || (cmd == IDREPLACEINSEL) || (cmd == IDREPLACEINBUF)) {
-		findWhat = GetDlgItemText2(hwndFR, IDFINDWHAT);
+		findWhat = dlg.ItemTextU(IDFINDWHAT);
 		props.Set("find.what", findWhat.c_str());
 		memFinds.Insert(findWhat.c_str());
-		wholeWord = BST_CHECKED ==
-		            ::SendMessage(wWholeWord, BM_GETCHECK, 0, 0);
-		matchCase = BST_CHECKED ==
-		            ::SendMessage(wMatchCase, BM_GETCHECK, 0, 0);
-		regExp = BST_CHECKED ==
-		         ::SendMessage(wRegExp, BM_GETCHECK, 0, 0);
-		wrapFind = BST_CHECKED ==
-		           ::SendMessage(wWrap, BM_GETCHECK, 0, 0);
-		unSlash = BST_CHECKED ==
-		          ::SendMessage(wUnSlash, BM_GETCHECK, 0, 0);
-		findInStyle = BST_CHECKED ==
-		              ::SendMessage(wFindInStyle, BM_GETCHECK, 0, 0);
-		findStyle = atoi(GetDlgItemText2(hwndFR, IDFINDSTYLE).c_str());
+		wholeWord = dlg.Checked(IDWHOLEWORD);
+		matchCase = dlg.Checked(IDMATCHCASE);
+		regExp = dlg.Checked(IDREGEXP);
+		wrapFind = dlg.Checked(IDWRAP);
+		unSlash = dlg.Checked(IDUNSLASH);
+		findInStyle = dlg.Checked(IDFINDINSTYLE);
+		findStyle = atoi(dlg.ItemTextU(IDFINDSTYLE).c_str());
 	}
 	if ((cmd == IDREPLACE) || (cmd == IDREPLACEALL) || (cmd == IDREPLACEINSEL) || (cmd == IDREPLACEINBUF)) {
-		replaceWhat = GetDlgItemText2(hwndFR, IDREPLACEWITH);
+		replaceWhat = dlg.ItemTextU(IDREPLACEWITH);
 		memReplaces.Insert(replaceWhat.c_str());
 	}
 
@@ -986,7 +952,7 @@ BOOL SciTEWin::HandleReplaceCommand(int cmd) {
 	}
 	char replDone[10];
 	sprintf(replDone, "%d", replacements);
-	::SetDlgItemText(hwndFR, IDREPLDONE, replDone);
+	dlg.SetItemText(IDREPLDONE, replDone);
 
 	return TRUE;
 }
@@ -995,40 +961,28 @@ BOOL SciTEWin::ReplaceMessage(HWND hDlg, UINT message, WPARAM wParam) {
 	// Avoid getting dialog items before set up or during tear down.
 	if (WM_SETFONT == message || WM_NCDESTROY == message)
 		return FALSE;
-	HWND wFindWhat = ::GetDlgItem(hDlg, IDFINDWHAT);
+	Dialog dlg(hDlg);
 	HWND wReplaceWith = ::GetDlgItem(hDlg, IDREPLACEWITH);
-	HWND wWholeWord = ::GetDlgItem(hDlg, IDWHOLEWORD);
-	HWND wMatchCase = ::GetDlgItem(hDlg, IDMATCHCASE);
-	HWND wRegExp = ::GetDlgItem(hDlg, IDREGEXP);
-	HWND wWrap = ::GetDlgItem(hDlg, IDWRAP);
-	HWND wUnSlash = ::GetDlgItem(hDlg, IDUNSLASH);
-	HWND wFindInStyle = ::GetDlgItem(hDlg, IDFINDINSTYLE);
-	HWND wFindStyle = ::GetDlgItem(hDlg, IDFINDSTYLE);
 
 	switch (message) {
 
 	case WM_INITDIALOG:
 		LocaliseDialog(hDlg);
-		FillComboFromMemory(wFindWhat, memFinds);
-		SetDlgItemText2(hDlg, IDFINDWHAT, findWhat.c_str());
-		FillComboFromMemory(wReplaceWith, memReplaces);
-		SetDlgItemText2(hDlg, IDREPLACEWITH, replaceWhat.c_str());
-		if (wholeWord)
-			::SendMessage(wWholeWord, BM_SETCHECK, BST_CHECKED, 0);
-		if (matchCase)
-			::SendMessage(wMatchCase, BM_SETCHECK, BST_CHECKED, 0);
-		if (regExp)
-			::SendMessage(wRegExp, BM_SETCHECK, BST_CHECKED, 0);
-		if (wrapFind)
-			::SendMessage(wWrap, BM_SETCHECK, BST_CHECKED, 0);
-		if (unSlash)
-			::SendMessage(wUnSlash, BM_SETCHECK, BST_CHECKED, 0);
-		if (findInStyle)
-			::SendMessage(wFindInStyle, BM_SETCHECK, BST_CHECKED, 0);
-		else
-			::EnableWindow(wFindStyle, false);
+		dlg.FillComboFromMemory(IDFINDWHAT, memFinds);
+		dlg.SetItemTextU(IDFINDWHAT, findWhat.c_str());
+		::SendDlgItemMessage(hDlg, IDFINDWHAT, CB_LIMITTEXT, CTL_TEXT_BUF - 1, 0);
+		dlg.FillComboFromMemory(IDREPLACEWITH, memReplaces);
+		dlg.SetItemTextU(IDREPLACEWITH, replaceWhat.c_str());
+		::SendDlgItemMessage(hDlg, IDREPLACEWITH, CB_LIMITTEXT, CTL_TEXT_BUF - 1, 0);
+		dlg.SetCheck(IDWHOLEWORD, wholeWord);
+		dlg.SetCheck(IDMATCHCASE, matchCase);
+		dlg.SetCheck(IDREGEXP, regExp);
+		dlg.SetCheck(IDWRAP, wrapFind);
+		dlg.SetCheck(IDUNSLASH, unSlash);
+		dlg.SetCheck(IDFINDSTYLE, findInStyle);
+		dlg.Enable(IDFINDSTYLE, findInStyle);
 		::SetDlgItemInt(hDlg, IDFINDSTYLE, SendEditor(SCI_GETSTYLEAT, SendEditor(SCI_GETCURRENTPOS)), FALSE);
-		::SetDlgItemText(hDlg, IDREPLDONE, "0");
+		dlg.SetItemText(IDREPLDONE, "0");
 		if (findWhat.length() != 0 && props.GetInt("find.replacewith.focus", 1)) {
 			::SetFocus(wReplaceWith);
 			return FALSE;
@@ -1047,7 +1001,8 @@ BOOL SciTEWin::ReplaceMessage(HWND hDlg, UINT message, WPARAM wParam) {
 			wFindReplace.Destroy();
 			return FALSE;
 		} else if (ControlIDOfCommand(wParam) == IDFINDINSTYLE) {
-			::EnableWindow(wFindStyle, BST_CHECKED == ::SendMessage(wFindInStyle, BM_GETCHECK, 0, 0));
+			findInStyle = dlg.Checked(IDFINDINSTYLE);
+			dlg.Enable(IDFINDSTYLE, findInStyle);
 			return TRUE;
 		} else {
 			return HandleReplaceCommand(ControlIDOfCommand(wParam));
@@ -1071,7 +1026,7 @@ BOOL SciTEWin::IncrementFindMessage(HWND hDlg, UINT message, WPARAM wParam) {
 	if (WM_SETFONT == message || WM_NCDESTROY == message)
 		return FALSE;
 
-	HWND wFindWhat = ::GetDlgItem(hDlg, IDC_INCFINDTEXT);
+	Dialog dlg(hDlg);
 
 	switch (message) {
 
@@ -1079,7 +1034,7 @@ BOOL SciTEWin::IncrementFindMessage(HWND hDlg, UINT message, WPARAM wParam) {
 		wFindIncrement = hDlg;
 		LocaliseDialog(hDlg);
 		SetWindowLong(hDlg, GWL_STYLE, WS_TABSTOP || GetWindowLong(hDlg, GWL_STYLE));
-		SetDlgItemText2(hDlg, IDC_INCFINDTEXT, ""); //findWhat.c_str()
+		dlg.SetItemTextU(IDC_INCFINDTEXT, ""); //findWhat.c_str()
 		SetFocus(hDlg);
 
 		PRectangle aRect = wFindIncrement.GetPosition();
@@ -1123,7 +1078,7 @@ BOOL SciTEWin::IncrementFindMessage(HWND hDlg, UINT message, WPARAM wParam) {
 			|| (ControlIDOfCommand(wParam) == IDC_INCFINDBTNOK)) {
 			SString ffLastWhat;
 			ffLastWhat = findWhat;
-			findWhat = GetDlgItemText2(hDlg, IDC_INCFINDTEXT);
+			findWhat = dlg.ItemTextU(IDC_INCFINDTEXT);
 
 			if (ControlIDOfCommand(wParam) != IDC_INCFINDBTNOK) {
 				CharacterRange cr = GetSelection();
@@ -1138,8 +1093,8 @@ BOOL SciTEWin::IncrementFindMessage(HWND hDlg, UINT message, WPARAM wParam) {
 				// Could not find string with added character so revert to previous value.
 				findWhat = ffLastWhat;
 				entered = true;
-				SetDlgItemText2(hDlg, IDC_INCFINDTEXT, findWhat.c_str());
-				SendMessage(wFindWhat, EM_SETSEL, ffLastWhat.length(), ffLastWhat.length());
+				dlg.SetItemTextU(IDC_INCFINDTEXT, findWhat.c_str());
+				SendMessage(dlg.Item(IDC_INCFINDTEXT), EM_SETSEL, ffLastWhat.length(), ffLastWhat.length());
 				entered = false;
 			}
 			return FALSE;
@@ -1242,28 +1197,55 @@ void SciTEWin::PerformGrep() {
 		flags += jobHasInput;
 	}
 
-	AddCommand(props.GetNewExpand("find.command"),
-		   props.Get("find.directory"),
-		   jobCLI, findInput, flags);
+	SString findCommand = props.GetNewExpand("find.command");
+	if (findCommand == "") {
+		// Call InternalGrep in a new thread
+		// searchParams is "(w|~)(c|~)\0files\0text"
+		// A "w" indicates whole word and a "c" indicates case sensitive
+		SString searchParams;
+		searchParams.append(wholeWord ? "w" : "~");
+		searchParams.append(matchCase ? "c" : "~");
+		searchParams.append("\0", 1);
+		searchParams.append(props.Get("find.files").c_str());
+		searchParams.append("\0", 1);
+		searchParams.append(props.Get("find.what").c_str());
+		AddCommand(searchParams, props.Get("find.directory"), jobGrep, findInput, flags);
+	} else {
+		AddCommand(findCommand,
+			   props.Get("find.directory"),
+			   jobCLI, findInput, flags);
+	}
 	if (commandCurrent > 0) {
 		Execute();
 	}
 }
 
-void SciTEWin::FillCombos(HWND hDlg) {
-	FillComboFromMemory(::GetDlgItem(hDlg, IDFINDWHAT), memFinds, true);
-	FillComboFromMemory(::GetDlgItem(hDlg, IDFILES), memFiles, true);
-	FillComboFromMemory(::GetDlgItem(hDlg, IDDIRECTORY), memDirectory, true);
+void SciTEWin::FillCombos(Dialog &dlg) {
+	dlg.FillComboFromMemory(IDFINDWHAT, memFinds, true);
+	dlg.FillComboFromMemory(IDFILES, memFiles, true);
+	dlg.FillComboFromMemory(IDDIRECTORY, memDirectory, true);
 }
 
 BOOL SciTEWin::GrepMessage(HWND hDlg, UINT message, WPARAM wParam) {
+	if (WM_SETFONT == message || WM_NCDESTROY == message)
+		return FALSE;
+	Dialog dlg(hDlg);
+
 	switch (message) {
 
 	case WM_INITDIALOG:
 		LocaliseDialog(hDlg);
-		FillCombos(hDlg);
-		::SetDlgItemText(hDlg, IDFINDWHAT, props.Get("find.what").c_str());
-		::SetDlgItemText(hDlg, IDDIRECTORY, props.Get("find.directory").c_str());
+		FillCombos(dlg);
+		dlg.SetItemText(IDFINDWHAT, props.Get("find.what").c_str());
+		dlg.SetItemText(IDDIRECTORY, props.Get("find.directory").c_str());
+		if (props.GetNewExpand("find.command") == "") {
+			// Empty means use internal that can respond to flags 
+			dlg.SetCheck(IDWHOLEWORD, wholeWord);
+			dlg.SetCheck(IDMATCHCASE, matchCase);
+		} else {
+			dlg.Enable(IDWHOLEWORD, false);
+			dlg.Enable(IDMATCHCASE, false);
+		}
 		return TRUE;
 
 	case WM_CLOSE:
@@ -1277,19 +1259,22 @@ BOOL SciTEWin::GrepMessage(HWND hDlg, UINT message, WPARAM wParam) {
 			return FALSE;
 
 		} else if (ControlIDOfCommand(wParam) == IDOK) {
-			findWhat = GetItemText(hDlg, IDFINDWHAT);
+			findWhat = dlg.ItemText(IDFINDWHAT);
 			props.Set("find.what", findWhat.c_str());
 			memFinds.Insert(findWhat.c_str());
 
-			SString files = GetItemText(hDlg, IDFILES);
+			SString files = dlg.ItemText(IDFILES);
 			props.Set("find.files", files.c_str());
 			memFiles.Insert(files.c_str());
 
-			SString directory = GetItemText(hDlg, IDDIRECTORY);
+			SString directory = dlg.ItemText(IDDIRECTORY);
 			props.Set("find.directory", directory.c_str());
 			memDirectory.Insert(directory.c_str());
 
-			FillCombos(hDlg);
+			wholeWord = dlg.Checked(IDWHOLEWORD);
+			matchCase = dlg.Checked(IDMATCHCASE);
+
+			FillCombos(dlg);
 
 			PerformGrep();
 			if (props.GetInt("find.in.files.close.on.find", 1)) {
@@ -1300,13 +1285,9 @@ BOOL SciTEWin::GrepMessage(HWND hDlg, UINT message, WPARAM wParam) {
 				return FALSE;
 			}
 		} else if (ControlIDOfCommand(wParam) == IDDOTDOT) {
-
-			SString directory = GetItemText(hDlg, IDDIRECTORY);
-			const char *lastSlash = strrchr(directory.c_str(), pathSepChar);
-			if (lastSlash) {
-				directory.remove(lastSlash - directory.c_str(), 0);
-				::SetDlgItemText(hDlg, IDDIRECTORY, directory.c_str());
-			}
+			FilePath directory(dlg.ItemText(IDDIRECTORY).c_str());
+			directory = directory.Directory();
+			dlg.SetItemText(IDDIRECTORY, directory.AsInternal());
 
 		} else if (ControlIDOfCommand(wParam) == IDBROWSE) {
 
@@ -1331,7 +1312,7 @@ BOOL SciTEWin::GrepMessage(HWND hDlg, UINT message, WPARAM wParam) {
 				info.lpszTitle = title.c_str();
 				info.ulFlags = 0;
 				info.lpfn = BrowseCallbackProc;
-				SString directory = GetItemText(hDlg, IDDIRECTORY);
+				SString directory = dlg.ItemText(IDDIRECTORY);
 				if (!directory.endswith(pathSepString)) {
 					directory += pathSepString;
 				}
@@ -1348,7 +1329,7 @@ BOOL SciTEWin::GrepMessage(HWND hDlg, UINT message, WPARAM wParam) {
 					char szDir[MAX_PATH];
 					if (::SHGetPathFromIDList(pidl, szDir)) {
 						// Set edit control to the directory path.
-						::SetDlgItemText(hDlg, IDDIRECTORY, szDir);
+						dlg.SetItemText(IDDIRECTORY, szDir);
 					}
 					pShellMalloc->Free(pidl);
 				}
@@ -1369,9 +1350,8 @@ void SciTEWin::FindInFiles() {
 		return;
 	SelectionIntoFind();
 	props.Set("find.what", findWhat.c_str());
-	char findInDir[1024];
-	GetDocumentDirectory(findInDir, sizeof(findInDir));
-	props.Set("find.directory", findInDir);
+	FilePath findInDir = filePath.Directory();
+	props.Set("find.directory", findInDir.AsFileSystem());
 	wFindInFiles = ::CreateDialogParam(hInstance, "Grep", MainHWND(), 
 		reinterpret_cast<DLGPROC>(GrepDlg), reinterpret_cast<sptr_t>(this));
 	wFindInFiles.Show();
@@ -1611,13 +1591,14 @@ BOOL SciTEWin::ParametersMessage(HWND hDlg, UINT message, WPARAM wParam) {
 	case WM_INITDIALOG: {
 			LocaliseDialog(hDlg);
 			wParameters = hDlg;
+			Dialog dlg(hDlg);
 			if (modalParameters) {
-				::SetDlgItemText(hDlg, IDCMD, parameterisedCommand.c_str());
+				dlg.SetItemText(IDCMD, parameterisedCommand.c_str());
 			}
 			for (int param = 0; param < maxParam; param++) {
 				SString paramText(param + 1);
 				SString paramTextVal = props.Get(paramText.c_str());
-				::SetDlgItemText(hDlg, IDPARAMSTART + param, paramTextVal.c_str());
+				dlg.SetItemText(IDPARAMSTART + param, paramTextVal.c_str());
 			}
 		}
 		return TRUE;
@@ -1686,6 +1667,45 @@ int SciTEWin::WindowMessageBox(Window &w, const SString &msg, int style) {
 	return ret;
 }
 
+void SciTEWin::FindMessageBox(const SString &msg, const SString *findItem) {
+
+	if (findItem == 0) {
+		SString msgBuf = LocaliseMessage(msg.c_str());
+		WindowMessageBox(wFindReplace.Created() ? wFindReplace : wSciTE, msgBuf, MB_OK | MB_ICONWARNING);
+	} else {
+		if (IsWindowsNT()) {
+
+			SString sFormat = LocaliseString(msg.c_str());
+			SString sPart1 = sFormat.substr(0, sFormat.search("^0", 0));
+			SString sPart2 = sFormat.substr(sFormat.search("^0", 0));
+			sPart2 = sPart2.substr(2); // skip initial ^0
+
+			WCHAR wszPart1[256];
+			::MultiByteToWideChar(CP_ACP, 0, sPart1.c_str(), -1, wszPart1, 256);
+
+			WCHAR wszPart2[256];
+			::MultiByteToWideChar(CP_ACP, 0, sPart2.c_str(), -1, wszPart2, 256);
+
+			WCHAR wszFindItem[CTL_TEXT_BUF];
+			::MultiByteToWideChar(CP_UTF8, 0, findItem->c_str(), -1, wszFindItem, CTL_TEXT_BUF);
+
+			WCHAR wszAppName[64];
+			::MultiByteToWideChar(CP_ACP, 0, appName, -1, wszAppName, 64);
+
+			WCHAR wszOutput[1024];
+			::lstrcpyW(wszOutput, wszPart1);
+			::lstrcatW(wszOutput, wszFindItem);
+			::lstrcatW(wszOutput, wszPart2);
+			::MessageBoxW(
+			    wFindReplace.Created() ? (HWND)wFindReplace.GetID() : (HWND)wSciTE.GetID(),
+			    wszOutput, wszAppName, MB_OK | MB_ICONWARNING);
+		} else {
+			SString msgBuf = LocaliseMessage(msg.c_str(), findItem->c_str());
+			WindowMessageBox(wFindReplace.Created() ? wFindReplace : wSciTE, msgBuf, MB_OK | MB_ICONWARNING);
+		}
+	}
+}
+
 BOOL SciTEWin::AboutMessage(HWND hDlg, UINT message, WPARAM wParam) {
 	switch (message) {
 
